@@ -304,7 +304,8 @@ static void exithandler(void)
 {
   int i;
   for (i=0;i<=cvsd_serversocketnum;i++)
-    close(cvsd_serversockets[i]);
+    if (close(cvsd_serversockets[i]))
+      log_log(LOG_WARNING,"problem closing server socket (ignored): %s",strerror(errno));
   log_log(LOG_INFO,"version %s bailing out",VERSION);
 }
 
@@ -355,7 +356,8 @@ static void handleconnection(int csock,const struct cvsd_cfg *cfg)
   log_log(LOG_DEBUG,"debug: fork() succeeded (child pid=%d)",(int)cpid);
 
   /* close the socket to prevent confusion */
-  close(csock);
+  if (close(csock))
+    log_log(LOG_WARNING,"problem closing connection socket in parent (ignored): %s",strerror(errno));
 }
 
 
@@ -413,7 +415,8 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
     if (setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&flag,sizeof(int))!=0)
     {
       log_log(LOG_ERR,"setsockopt(SO_REUSEADDR) failed: %s",strerror(errno));
-      close(sock);
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       exit(1);
     }
 
@@ -426,7 +429,8 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
     if (setsockopt(sock,SOL_SOCKET,SO_LINGER,&**,sizeof(**))!=0)
     {
       log_log(LOG_ERR,"setsockopt(SO_LINGER) failed: %s",strerror(errno));
-      close(sock);
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       exit(1);
     }
 #endif /* TODO */
@@ -435,13 +439,15 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
     if ((i=fcntl(sock,F_GETFL,0))<0)
     {
       log_log(LOG_ERR,"fctnl(F_GETFL) failed: %s",strerror(errno));
-      close(sock);
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       exit(1);
     }
     if (fcntl(sock,F_SETFL,i|O_NONBLOCK)<0)
     {
       log_log(LOG_ERR,"fctnl(F_SETFL,O_NONBLOCK) failed: %s",strerror(errno));
-      close(sock);
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       exit(1);
     }
 
@@ -452,12 +458,14 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
           (last->ai_family==AF_INET6)&&
           (addr->ai_family==AF_INET))
       {
-        close(sock);
         log_log(LOG_DEBUG,"debug: bind() failed (ignored): %s",strerror(errno));
+        if (close(sock))
+          log_log(LOG_WARNING,"problem closing socket (ignored): %s",strerror(errno));
         continue;
       }
       log_log(LOG_ERR,"bind() failed: %s",strerror(errno));
-      close(sock);
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       exit(1);
     }
 
@@ -465,7 +473,8 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
     if (listen(sock,1)<0)
     {
       log_log(LOG_ERR,"listen() failed: %s",strerror(errno));
-      close(sock);
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       exit(1);
     }
 
@@ -473,7 +482,8 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
     if (cvsd_serversocketnum>=MAXSERVERSOCKETS)
     {
       log_log(LOG_ERR,"maximum number of listening serversockets reached: edit MAXSERVERSOCKETS in cvsd.c and recompile!");
-      close(sock);
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       exit(1);
     }
     cvsd_serversockets[cvsd_serversocketnum++]=sock;
@@ -573,11 +583,13 @@ static void acceptconnection(const struct cvsd_cfg *cfg)
         case 0: break;
         case EAI_SYSTEM:
           log_log(LOG_ERR,"getnameinfo() failed: %s",strerror(errno));
-          close(csock);
+          if (close(csock))
+            log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
           continue;
         default:
           log_log(LOG_ERR,"getnameinfo() failed: %s",gai_strerror(j));
-          close(csock);
+          if (close(csock))
+            log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
           continue;
       }
       /* convert IPv4-mapped-IPv6 address to an IPv4 one */
@@ -592,7 +604,8 @@ static void acceptconnection(const struct cvsd_cfg *cfg)
       if(!hosts_access(&rinfo))
       {
         log_log(LOG_INFO,"connection from %s %s refused by tcp wrappers",host,serv);
-        close(csock);
+        if (close(csock))
+          log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
         continue;
       }
 #endif /* USE_LIBWRAP */
@@ -604,13 +617,15 @@ static void acceptconnection(const struct cvsd_cfg *cfg)
       if ((j=fcntl(csock,F_GETFL,0))<0)
       {
         log_log(LOG_ERR,"fctnl(F_GETFL) failed: %s",strerror(errno));
-        close(csock);
+        if (close(csock))
+          log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
         exit(1);
       }
       if (fcntl(csock,F_SETFL,j&~O_NONBLOCK)<0)
       {
         log_log(LOG_ERR,"fctnl(F_SETFL,~O_NONBLOCK) failed: %s",strerror(errno));
-        close(csock);
+        if (close(csock))
+          log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
         exit(1);
       }
 
@@ -634,8 +649,16 @@ static void write_pidfile(const char *filename)
       log_log(LOG_ERR,"cannot open pid file (%s): %s",filename,strerror(errno));
       exit(1);
     }
-    fprintf(fp,"%d\n",(int)getpid());
-    fclose(fp);
+    if (fprintf(fp,"%d\n",(int)getpid())<=0)
+    {
+      log_log(LOG_ERR,"error writing pid file (%s)",filename);
+      exit(1);
+    }
+    if (fclose(fp))
+    {
+      log_log(LOG_ERR,"error writing pid file (%s): %s",filename,strerror(errno));
+      exit(1);
+    }
   }
 }
 
